/** @file
  * 
  * @brief    Source file of application-level BLE Advertising module
  *
  * @details  Include:
  *             Advertising module initialization and event handler
  *             Helper function to start advertising
  *             
  *
  * BLE stack events should be propagated to the SDK-level BLE Advertising module via the alias
  * function @ref fw_ble_advertising_on_ble_evt()
  *
  */

#include "fw_ble_advertising.h"

/** ===== Standard library includes ============================================================= */

/** ===== nRF5* SDK includes ==================================================================== */
#include "app_error.h"
#include "app_util.h"
#include "ble_advertising.h"
#include "nrf.h"

/** ===== Non-SDK and non-standard-library includes ============================================= */
#include "dev_config.h"


// The advertising interval (in units of 0.625 ms).
#define APP_ADV_INTERVAL                MSEC_TO_UNITS(CFG_ADVERTISING_INTERVAL_MILISECS, \
                                                        UNIT_0_625_MS)
// The advertising timeout (in units of seconds).
#define APP_ADV_TIMEOUT_IN_SECONDS      CFG_ADVERTISING_TIMEOUT_SECONDS 


/**@brief     Inline function serves as alias for @ref ble_advertising_on_ble_evt(p_ble_evt)
 * @param[in] p_ble_evt  The event received from the BLE stack.
 */
void fw_ble_advertising_on_ble_evt(ble_evt_t * p_ble_evt)
{
  ble_advertising_on_ble_evt(p_ble_evt);
  if (p_ble_evt->header.evt_id == BLE_GAP_EVT_DISCONNECTED)
  {
    ble_advertising_start(BLE_ADV_MODE_FAST);
  }
}


/**@brief     Inline function serves as alias for @ref ble_advertising_start(BLE_ADV_MODE_FAST)
 * @param[in] p_ble_evt  The event received from the BLE stack.
 */
uint32_t __inline fw_ble_advertising_start(void)
{
  return ble_advertising_start(BLE_ADV_MODE_FAST);
}


/**@brief Function for handling advertising events.
 *
 * @details This function will be called for advertising events which are passed to the application.
 *
 * @param[in] ble_adv_evt  Advertising event.
 */
static void on_adv_evt(ble_adv_evt_t ble_adv_evt)
{
}


/**@brief Function for initializing the Advertising functionality.
 */
void fw_ble_advertising_init(void)
{
  uint32_t                                  err_code;
  
  ble_advdata_t                             advdata;

  // Build advertising data struct to pass into @ref ble_advertising_init.
  memset(&advdata, 0, sizeof(advdata));
  
  // Device name
  advdata.name_type                         = BLE_ADVDATA_FULL_NAME;
  advdata.include_appearance                = false;
  advdata.flags                             = BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE;
  
  // Device's "serial number"
//  [0] = NRF_FICR->DEVICEADDR[0] & 0x000000FF;
//  [1] = (NRF_FICR->DEVICEADDR[0] >> 8) & 0x000000FF;
//  [2] = (NRF_FICR->DEVICEADDR[0] >> 16) & 0x000000FF;
//  [3] = (NRF_FICR->DEVICEADDR[0] >> 24) & 0x000000FF;
//  [4] = NRF_FICR->DEVICEADDR[1] & 0x000000FF;
//  [5] = ((NRF_FICR->DEVICEADDR[1] | 0x0000C000) >> 8) & 0x000000FF;

  ble_adv_modes_config_t options = {0};
  options.ble_adv_fast_enabled  = BLE_ADV_FAST_ENABLED;
  options.ble_adv_fast_interval = APP_ADV_INTERVAL;
  options.ble_adv_fast_timeout  = APP_ADV_TIMEOUT_IN_SECONDS;

  err_code = ble_advertising_init(&advdata, NULL, &options, on_adv_evt, NULL);
  // TODO: Implement error hanlder
  APP_ERROR_CHECK(err_code);
}
