/** @file
  * 
  * @brief    Source file of application-level implementation of GAP, Connection Parameter and BLE Stack
  *
  * @details  Include:
  *             GAP Parameters initialization
  *             Connection Parameter Module initialization and event handler implementation
  *             BLE Stack initialization and event handler implementation
  *
  */

#include "fw_ble_others.h"

/** ===== Standard library includes ============================================================= */
#include <string.h>

/** ===== nRF5* SDK includes ==================================================================== */
#include "app_error.h"
#include "app_timer.h"
#include "ble.h"
#include "nrf51.h"
#include "softdevice_handler.h"

/** ===== Non-SDK and non-standard-library includes ============================================= */
#include "dev_config.h"
#include "fw_ble_advertising.h"
#include "fw_ble_services.h"


// Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called.
#define FIRST_CONN_PARAMS_UPDATE_DELAY  APP_TIMER_TICKS(CFG_CONN_PARAMS_FIRST_UPDATE_DELAY_MILISECS, \
                                                        CFG_APP_TIMER_PRESCALER)
                                                        
// Time between each call to sd_ble_gap_conn_param_update after the first call.
#define NEXT_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(CFG_CONN_PARAMS_NEXT_UPDATE_DELAY_MILISECS, \
                                                        CFG_APP_TIMER_OP_QUEUE_SIZE) 
#define MAX_CONN_PARAMS_UPDATE_COUNT    3                                                   /**< Number of attempts before giving up the connection parameter negotiation. */

// Minimum acceptable connection interval. Connection interval uses 1.25 ms units.
#define MIN_CONN_INTERVAL               MSEC_TO_UNITS(CFG_MIN_CONN_INTERVAL_MILISECS, UNIT_1_25_MS)
// Maximum acceptable connection interval. Connection interval uses 1.25 ms units.
#define MAX_CONN_INTERVAL               MSEC_TO_UNITS(CFG_MAX_CONN_INTERVAL_MILISECS, UNIT_1_25_MS)
// Slave latency
#define SLAVE_LATENCY                   CFG_SLAVE_LATENCY  
// Connection supervisory timeout, Supervision Timeout uses 10 ms units.
#define CONN_SUP_TIMEOUT                MSEC_TO_UNITS(CFG_CONN_SUP_TIMEOUT_MILISECS, UNIT_10_MS)


static  uint16_t  m_conn_handle = BLE_CONN_HANDLE_INVALID;


/**@brief     Inline function as alias for @ref ble_conn_params_on_ble_evt(p_ble_evt)
 * @param[in] p_ble_evt  The event received from the BLE stack.
 */
void __inline fw_ble_others_conn_params_on_ble_evt(ble_evt_t * p_ble_evt)
{
  ble_conn_params_on_ble_evt(p_ble_evt);
}


/**@brief Function for handling an event from the Connection Parameters Module.
 *
 * @details This function will be called for all events in the Connection Parameters Module
 *          which are passed to the application.
 *
 * @note All this function does is to disconnect. This could have been done by simply setting
 *       the disconnect_on_fail config parameter, but instead we use the event handler
 *       mechanism to demonstrate its use.
 *
 * @param[in] p_evt  Event received from the Connection Parameters Module.
 */
static void fw_ble_others_on_conn_params_evt(ble_conn_params_evt_t * p_evt)
{
//    uint32_t err_code;
//    
  if(p_evt->evt_type == BLE_CONN_PARAMS_EVT_FAILED)
  {
    // TODO: Should not simply disconnect here. 
//      err_code = sd_ble_gap_disconnect(m_m_conn_handle, BLE_HCI_CONN_INTERVAL_UNACCEPTABLE);
//      APP_ERROR_CHECK(err_code);
    }
}


/**@brief Function for handling errors from the Connection Parameters module.
 *
 * @param[in] nrf_error  Error code containing information about what went wrong.
 */
static void fw_ble_others_conn_params_error_handler(uint32_t nrf_error)
{
  APP_ERROR_HANDLER(nrf_error);
}


/**@brief Function for initializing the Connection Parameters module.
 */
void fw_ble_others_conn_params_init(void)
{
  uint32_t               err_code;
  ble_conn_params_init_t cp_init;
  
  memset(&cp_init, 0, sizeof(cp_init));

  cp_init.p_conn_params                  = NULL;
  cp_init.first_conn_params_update_delay = FIRST_CONN_PARAMS_UPDATE_DELAY;
  cp_init.next_conn_params_update_delay  = NEXT_CONN_PARAMS_UPDATE_DELAY;
  cp_init.max_conn_params_update_count   = MAX_CONN_PARAMS_UPDATE_COUNT;
  cp_init.start_on_notify_cccd_handle    = BLE_GATT_HANDLE_INVALID;
  cp_init.disconnect_on_fail             = false;
  cp_init.evt_handler                    = fw_ble_others_on_conn_params_evt;
  cp_init.error_handler                  = fw_ble_others_conn_params_error_handler;
  
  err_code = ble_conn_params_init(&cp_init);
  APP_ERROR_CHECK(err_code);
}


/**@brief Function for the GAP initialization.
 *
 * @details This function will set up all the necessary GAP (Generic Access Profile) parameters of 
 *          the device. It also sets the permissions and appearance.
 */
void fw_ble_others_gap_params_init(void)
{
  uint32_t                err_code;
  ble_gap_conn_params_t   gap_conn_params;
  ble_gap_conn_sec_mode_t sec_mode;
  
  uint8_t device_name[20]; 
  
//  [0] = NRF_FICR->DEVICEADDR[0] & 0x000000FF;
//  [1] = (NRF_FICR->DEVICEADDR[0] >> 8) & 0x000000FF;
//  [2] = (NRF_FICR->DEVICEADDR[0] >> 16) & 0x000000FF;
//  [3] = (NRF_FICR->DEVICEADDR[0] >> 24) & 0x000000FF;
//  [4] = NRF_FICR->DEVICEADDR[1] & 0x000000FF;
//  [5] = ((NRF_FICR->DEVICEADDR[1] | 0x0000C000) >> 8) & 0x000000FF;
  
  sprintf((char*) device_name, 
            "nSMART COI %02x%02x%02x%02x",
            ((NRF_FICR->DEVICEADDR[1] | 0x0000C000) >> 8) & 0x000000FF,
            NRF_FICR->DEVICEADDR[1] & 0x000000FF,
            (NRF_FICR->DEVICEADDR[0] >> 24) & 0x000000FF,
            (NRF_FICR->DEVICEADDR[0] >> 16) & 0x000000FF
          );
  
  BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);
  
  err_code = sd_ble_gap_device_name_set(&sec_mode,
                                        (const uint8_t *) device_name,
                                        strlen((char*) device_name));
  APP_ERROR_CHECK(err_code);

  memset(&gap_conn_params, 0, sizeof(gap_conn_params));

  gap_conn_params.min_conn_interval = MIN_CONN_INTERVAL;
  gap_conn_params.max_conn_interval = MAX_CONN_INTERVAL;
  gap_conn_params.slave_latency     = SLAVE_LATENCY;
  gap_conn_params.conn_sup_timeout  = CONN_SUP_TIMEOUT;

  err_code = sd_ble_gap_ppcp_set(&gap_conn_params);
  APP_ERROR_CHECK(err_code);
}


/** @brief Weak function to be override with any BLE event processing that would be desirable
  *         elsewhere
  */
__WEAK void on_ble_evt(ble_evt_t * p_ble_evt)
{
}


/**@brief Function for dispatching a S110 SoftDevice event to all modules with a S110 SoftDevice 
 *        event handler.
 *
 * @details This function is called from the S110 SoftDevice event interrupt handler after a S110 
 *          SoftDevice event has been received.
 *
 * @param[in] p_ble_evt  S110 SoftDevice event.
 */
static void fw_ble_others_ble_evt_dispatch(ble_evt_t * p_ble_evt)
{
  uint32_t err_code;
  
  // Handle some event of interest first
  if (p_ble_evt->header.evt_id == BLE_GAP_EVT_CONNECTED)
  {
    // Acquire connection handle
    m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
    
    // Set System Attributes
    err_code = sd_ble_gatts_sys_attr_set(m_conn_handle, NULL, 0, 0);
    APP_ERROR_CHECK(err_code);
  }
  else if (p_ble_evt->header.evt_id == BLE_GAP_EVT_DISCONNECTED)
  {
    // Invalidate local connection handle
    m_conn_handle = BLE_CONN_HANDLE_INVALID;
  }
  else if (p_ble_evt->header.evt_id == BLE_GAP_EVT_SEC_PARAMS_REQUEST)
  {
    // Pairing not supported
    err_code = sd_ble_gap_sec_params_reply(m_conn_handle, BLE_GAP_SEC_STATUS_PAIRING_NOT_SUPP, NULL, NULL);
    APP_ERROR_CHECK(err_code);
  }
  
  
  // Dispatch to connection parameter module
  fw_ble_others_conn_params_on_ble_evt(p_ble_evt);
  
  // Dispatch to BLE services module
  fw_ble_services_on_ble_evt(p_ble_evt);
  
  // Dispatch to BLE advertising module
  fw_ble_advertising_on_ble_evt(p_ble_evt);
  
  // Dispatch to on_ble_evt()
  on_ble_evt(p_ble_evt);
}


/**@brief Function for the S110 SoftDevice initialization.
 *
 * @details This function initializes the S110 SoftDevice and the BLE event interrupt.
 */
void fw_ble_others_ble_stack_init(void)
{
  uint32_t err_code;
  
  // Initialize SoftDevice.
  SOFTDEVICE_HANDLER_INIT(NRF_CLOCK_LFCLKSRC_RC_250_PPM_4000MS_CALIBRATION, NULL);

  // Enable BLE stack.
  ble_enable_params_t ble_enable_params;
  memset(&ble_enable_params, 0, sizeof(ble_enable_params));

#if (defined(S130) || defined(S132))
  ble_enable_params.gatts_enable_params.attr_tab_size   = BLE_GATTS_ATTR_TAB_SIZE_DEFAULT;
#endif

  ble_enable_params.gatts_enable_params.service_changed = CFG_IS_SRVC_CHANGED_CHARACT_PRESENT;
  err_code = sd_ble_enable(&ble_enable_params);
  APP_ERROR_CHECK(err_code);
  
  // Subscribe for BLE events.
  err_code = softdevice_ble_evt_handler_set(fw_ble_others_ble_evt_dispatch);
  APP_ERROR_CHECK(err_code);
}




