/** @file
  * 
  * @brief    Source file of user implemented error handler to be used with nRF5 SDK
  *
  * @details  If the macro @ref DEBUG is defined, the error handler will trap the application flow in
  *           a infinite loop. The location where the error is asserted will be stored in the variables
  *           @ref m_error_code, @ref m_line_num and @ref m_p_file_name
  *
  */

/** ===== Standard library includes ============================================================= */
#include <stdbool.h>
#include <stdint.h>

/** ===== nRF5* SDK includes ==================================================================== */
#include "nrf.h"
#include "nordic_common.h"

/** ===== Non-SDK and non-standard-library includes ============================================= */


// Value used as error code on stack dump, can be used to identify stack location on stack unwind.
#define DEAD_BEEF   0xDEADBEEF                                  


#ifdef DEBUG // ------------------------------------------------------------------------------------

/* global error variables - in order to prevent removal by optimizers */
static uint32_t m_error_code;
static uint32_t m_line_num;
const uint8_t * m_p_file_name;

#endif // #ifdef DEBUG -----------------------------------------------------------------------------

/**@brief Function for error handling, which is called when an error has occurred.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyze
 *          how your product is supposed to react in case of error.
 *
 * @param[in] error_code  Error code supplied to the handler.
 * @param[in] line_num    Line number where the handler is called.
 * @param[in] p_file_name Pointer to the file name.
 *
 */
void app_error_handler(uint32_t error_code, uint32_t line_num, const uint8_t * p_file_name)
{
    // On assert, the system can only recover with a reset.
#ifndef DEBUG
    NVIC_SystemReset();
#else

    // The following variable helps Keil keep the call stack visible, in addition, it can be set to
    // 0 in the debugger to continue executing code after the error check.
    volatile bool loop = true;
    UNUSED_VARIABLE(loop);

    m_error_code = error_code;
    m_line_num = line_num;
    m_p_file_name = p_file_name;

    UNUSED_VARIABLE(m_error_code);
    UNUSED_VARIABLE(m_line_num);
    UNUSED_VARIABLE(m_p_file_name);
    __disable_irq();

    while(loop);

#endif // DEBUG
}


/**@brief Function for assert macro callback.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyse 
 *          how your product is supposed to react in case of Assert.
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in] line_num    Line number of the failing ASSERT call.
 * @param[in] p_file_name File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}


